package com.ruoyi.common.utils.map;

import com.alibaba.fastjson2.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.experimental.UtilityClass;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.geotools.data.DataUtilities;
import org.geotools.data.collection.ListFeatureCollection;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.simple.SimpleFeatureBuilder;
import org.geotools.geojson.feature.FeatureJSON;
import org.geotools.geojson.geom.GeometryJSON;
import org.geotools.geometry.jts.JTS;
import org.geotools.referencing.CRS;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.locationtech.jts.geom.*;
import org.locationtech.jts.geom.Point;
import org.locationtech.jts.io.WKTReader;
import org.locationtech.jts.io.WKTWriter;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.referencing.operation.MathTransform;

import java.io.File;
import java.util.Collection;
import java.util.Collections;
import java.util.List;


/**
 * GeoUtils.
 *
 * @author qiuzhenzhao
 * @date 2024/04/19
 */
@Slf4j
@UtilityClass
public class GeoUtils {
    private static final GeometryFactory GEOMETRY_FACTORY = new GeometryFactory();
    private static final String AUTO = "AUTO:42001,%s,%s";
    private static final WKTWriter WKT_WRITER = new WKTWriter();
    private static final WKTReader WKT_READER = new WKTReader();
    private static final GeometryJSON GEOMETRY_JSON = new GeometryJSON(15);
    private final static FeatureJSON FEATURE_JSON = new FeatureJSON(GEOMETRY_JSON);

    private static final double EARTH_RADIUS = 6371000; // 地球半径，单位：米

    /**
     * 合并多边形  Geographic
     * */
    public Geometry union(Collection<Geometry> geometries) {
        return GEOMETRY_FACTORY.buildGeometry(geometries).union();
    }

    /**
     * geometry转geojson
     * */
    @SneakyThrows
    public String geoJson(Geometry geometry) {
        SimpleFeatureType type = DataUtilities.createType("Link", "geometry:" + geometry.getGeometryType());
        SimpleFeatureBuilder featureBuilder = new SimpleFeatureBuilder(type);
        featureBuilder.add(geometry);
        SimpleFeature feature = featureBuilder.buildFeature(null);
        FeatureCollection<?, ?> featureCollection = new ListFeatureCollection(type, Collections.singletonList(feature));
        return FEATURE_JSON.toString(featureCollection);
    }

    /**
     *  定的geometry对象转换为GeoJSON格式，并将其写入到指定的file中
     * */
    @SneakyThrows
    public void geoJson(Geometry geometry, File file) {
        SimpleFeatureType type = DataUtilities.createType("Link", "geometry:" + geometry.getGeometryType());
        SimpleFeatureBuilder featureBuilder = new SimpleFeatureBuilder(type);
        featureBuilder.add(geometry);
        SimpleFeature feature = featureBuilder.buildFeature(null);
        FeatureCollection<?, ?> featureCollection = new ListFeatureCollection(type, Collections.singletonList(feature));
        FEATURE_JSON.writeFeatureCollection(featureCollection, FileUtils.openOutputStream(file));
    }
    /**
     *  wkt转Geometry对象
     *  wkt格式：POLYGON((x1 y1,x2 y2,x3 y3,x4 y4,x1 y1))
     * */
    public Geometry fromWkt(String wkt) {
        if (StringUtils.isBlank(wkt)) {
            return null;
        }
        try {
            return WKT_READER.read(wkt);
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            return null;
        }
    }
    /**
     *  json转Geometry对象
     *  格式：[[120.356267,26.941506],[120.357168,26.941262],[120.35697,26.940564]]
     * */
    public Geometry toGeometry(String json) {
        if (StringUtils.isBlank(json)) {
            return null;
        }
        try {
            // 将json转成二位数组并形成闭合
            double[][] coordinates = transform(json);
            log.info("coordinates:{}", JSON.toJSONString(coordinates));
            // 构建坐标数组
            Coordinate[] points = new Coordinate[coordinates.length];
            for (int i = 0; i < coordinates.length; i++) {
                double lon = coordinates[i][0];
                double lat = coordinates[i][1];
                points[i] = new Coordinate(lon, lat);
            }
            // 创建LinearRing对象
            LinearRing linearRing = GEOMETRY_FACTORY.createLinearRing(points);
            // 创建Polygon对象
            Polygon polygon = GEOMETRY_FACTORY.createPolygon(linearRing);
            return polygon;
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            return null;
        }
    }

    /**
     *  将json转成二位数组
     * */
    public double[][] transform(String json) {
        // 使用 Jackson 库解析 JSON 字符串为二维数组
        ObjectMapper objectMapper = new ObjectMapper();
        double[][] coordinates;
        try {
            coordinates = objectMapper.readValue(json, double[][].class);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        // 添加闭合多边形的最后一个点
        int numPoints = coordinates.length;
        double[][] closedCoordinates = new double[numPoints + 1][2];
        for (int i = 0; i < numPoints; i++) {
            closedCoordinates[i] = coordinates[i];
        }
        closedCoordinates[numPoints] = coordinates[0]; // 将第一个点复制到最后一个点
        return closedCoordinates;
    }



    /**
     *  Geometry对象转wkt
     * */
    public String wkt(Geometry geometry) {
        if (geometry == null) {
            return null;
        }
        try {
            return WKT_WRITER.write(geometry);
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            return null;
        }
    }
    /**
     *  JSON转Geometry
     * */
    public Geometry fromJson(String json) {
        if (StringUtils.isBlank(json)) {
            return null;
        }
        try {
            return GEOMETRY_JSON.read(json);
        } catch (Exception e) {
            log.warn(e.getMessage(), e);
            return null;
        }
    }
    /**
     * 计算Geometry的面积
     * */
    @SneakyThrows
    public Double calcArea(Geometry geometry) {
        Point centroid = geometry.getCentroid();
        MathTransform mathTransform = CRS.findMathTransform(DefaultGeographicCRS.WGS84,
                CRS.decode(String.format(AUTO, centroid.getX(), centroid.getY())));
        Geometry transform = JTS.transform(geometry, mathTransform);
        return transform.getArea();
    }

    /**
     * 判断经纬度是否在多边形区域内
     * */
    public boolean isInCircle(String longitude, String latitude,Geometry polygon){
        /**创建一个点*/
        GeometryFactory geometryFactory = new GeometryFactory();
        Coordinate coordinate = new Coordinate(Double.parseDouble(longitude), Double.parseDouble(latitude));
        Point point = geometryFactory.createPoint(coordinate);
        return polygon.contains(point);
    }

    /**
     * 判断一个点是否在一个圆形区域内（考虑误差距离）
     * */
    public static boolean isInPolygonWithTolerance (String longitude, String latitude, Geometry polygon, double tolerance) {
        double lon = Double.parseDouble(longitude);
        double lat = Double.parseDouble(latitude);

        GeometryFactory geometryFactory = new GeometryFactory();
        Coordinate coordinate = new Coordinate(lon, lat);
        Point point = geometryFactory.createPoint(coordinate);

        if (polygon.contains(point)) {
            return true;
        } else {
            // 获取多边形的外边界
            Coordinate[] coordinates = polygon.getCoordinates();
            for (Coordinate coord : coordinates) {
                double distance = calculateDistance(lat, lon, coord.y, coord.x);
                if (distance <= tolerance) {
                    return true;
                }
            }
            return false;
        }
    }

    // 将角度转换为弧度
    private static double deg2rad(double deg) {
        return deg * (Math.PI / 180);
    }

    // 使用 Haversine 公式计算两点间的距离
    public static double calculateDistance(double lat1, double lon1, double lat2, double lon2) {
        double dLat = deg2rad(lat2 - lat1);
        double dLon = deg2rad(lon1 - lon2); // 修正此处为 (lon1 - lon2)
        double a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
                Math.cos(deg2rad(lat1)) * Math.cos(deg2rad(lat2)) *
                        Math.sin(dLon / 2) * Math.sin(dLon / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        return EARTH_RADIUS * c;
    }

    /**
     * 判获取到最近一个运营区
     * */
    public static boolean getNearestOperatingArea (String longitude, String latitude, List<Geometry> polygon) {
        double lon = Double.parseDouble(longitude);
        double lat = Double.parseDouble(latitude);

        GeometryFactory geometryFactory = new GeometryFactory();
        Coordinate coordinate = new Coordinate(lon, lat);
        Point point = geometryFactory.createPoint(coordinate);

        for (Geometry geometry : polygon) {
            if (geometry.contains(point)) {
                return true;
            }else{

            }
        }
        return false;
    }

}
