package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.EtOrderDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.common.utils.bean.BeanValidators;
import com.ruoyi.common.utils.verify.VerifyIdentityUtil;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.vo.AuthenticationVo;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.service.IAsUserService;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.IEtOrderService;
import com.ruoyi.system.service.ISysConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.Validator;
import java.math.BigDecimal;
import java.util.List;

/**
 * 用户 业务层处理
 *
 * @author ruoyi
 */
@Service
public class AsUserServiceImpl implements IAsUserService
{
    private static final Logger log = LoggerFactory.getLogger(AsUserServiceImpl.class);

    @Resource
    private AsUserMapper asUserMapper;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    protected Validator validator;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Value("${et.verifyUrl}")
    private String verifyUrl;

    @Value("${et.appcode}")
    private String appcode;

    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    public List<AsUser> selectUserList(AsUser user)
    {
        /** 获取设备绑定数量*/
        List<AsUser> users = asUserMapper.selectUserList(user);
        for(AsUser u:users){
            EtOrder etOrder  = new EtOrder();
            etOrder.setUserId(u.getUserId());
            EtOrder order = etOrderService.latestOrder(etOrder);
            if(ObjectUtil.isNotNull(order)){
                EtOrderDto etOrderVo = new EtOrderDto();
                BeanUtils.copyBeanProp(etOrderVo,order);
                u.setLatestOrder(etOrderVo);
            }
        }
        return users;
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<AsUser> selectAllocatedList(AsUser user)
    {
        return asUserMapper.selectAllocatedList(user);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<AsUser> selectUnallocatedList(AsUser user)
    {
        return asUserMapper.selectUnallocatedList(user);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public AsUser selectUserByUserName(String userName)
    {
        return asUserMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public AsUser selectUserById(Long userId)
    {
        AsUser users = asUserMapper.selectUserById(userId);
        if(ObjectUtil.isNull(users)){
            throw new ServiceException("没有该用户："+userId);
        }
        EtOrder etOrder  = new EtOrder();
        etOrder.setUserId(users.getUserId());
        EtOrder order = etOrderService.latestOrder(etOrder);
        if(ObjectUtil.isNotNull(order)){
            EtOrderDto etOrderVo = new EtOrderDto();
            BeanUtils.copyBeanProp(etOrderVo,order);
            users.setLatestOrder(etOrderVo);
        }
        return users;
    }


    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean checkUserNameUnique(AsUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        AsUser info = asUserMapper.checkUserNameUnique(user.getUserName());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkPhoneUnique(AsUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        AsUser info = asUserMapper.checkPhoneUnique(user.getPhonenumber());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkEmailUnique(AsUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        AsUser info = asUserMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }


    /**
     * 新增保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertUser(AsUser user)
    {
        return asUserMapper.insertUser(user);
    }

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean registerUser(AsUser user)
    {
        return asUserMapper.insertUser(user) > 0;
    }

    /**
     * 修改保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateUser(AsUser user)
    {
        Long userId = user.getUserId();
        return asUserMapper.updateUser(user);
    }


    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(AsUser user)
    {
        return asUserMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(AsUser user)
    {
        return asUserMapper.updateUser(user);
    }

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatar(String userName, String avatar)
    {
        return asUserMapper.updateUserAvatar(userName, avatar) > 0;
    }

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(AsUser user)
    {
        return asUserMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password)
    {
        return asUserMapper.resetUserPwd(userName, password);
    }


    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserById(Long userId)
    {
        return asUserMapper.deleteUserById(userId);
    }

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserByIds(Long[] userIds)
    {
        return asUserMapper.deleteUserByIds(userIds);
    }

    /**
     * 导入用户数据
     *
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    @Override
    public String importUser(List<AsUser> userList, Boolean isUpdateSupport, String operName)
    {
        if (StringUtils.isNull(userList) || userList.size() == 0)
        {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String password = configService.selectConfigByKey("sys.user.initPassword");
        for (AsUser user : userList)
        {
            try
            {
                // 验证是否存在这个用户
                AsUser u = asUserMapper.selectUserByUserName(user.getUserName());
                if (StringUtils.isNull(u))
                {
                    BeanValidators.validateWithException(validator, user);
                    user.setPassword(SecurityUtils.encryptPassword(password));
                    user.setCreateBy(operName);
                    asUserMapper.insertUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    BeanValidators.validateWithException(validator, user);
                    user.setUserId(u.getUserId());
                    user.setUpdateBy(operName);
                    asUserMapper.updateUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getUserName() + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 通过手机号查询用户
     *
     * @param phone 手机号
     * @return 用户对象信息
     */
    @Override
    public AsUser selectUserByPhone(String phone) {
        return asUserMapper.selectUserByPhone(phone);
    }

    /**
     * 通过openid查询用户
     *
     * @param openid 手机号
     * @return 用户对象信息
     */
    @Override
    public AsUser selectUserByWxopenid(String openid) {
        return asUserMapper.selectUserByWxopenid(openid);
    }

    /**
     * 检测用户是否有充值过押金
     * @param userId
     */
    @Override
    public Boolean checkIsDeposit(Long userId) {
        AsUser asUser = asUserMapper.selectUserById(userId);
        BigDecimal balance = asUser.getBalance();
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(asUser.getAreaId());
        BigDecimal deposit = new BigDecimal(area.getDeposit());
        if(ObjectUtil.isNull(balance)){
            log.info("用户【{}】余额为空",userId);
            return false;
        }
        if(balance.compareTo(BigDecimal.ZERO)==0 || balance.compareTo(deposit)<0){//余额为0或者押金和余额不相等
            log.info("用户【{}】余额为0或者押金小于余额不相等",userId);
            return false;
        }
        // 判断是否有押金充值记录
        EtOrder etOrder = new EtOrder();
        etOrder.setType(ServiceConstants.ORDER_TYPE_DEPOSIT);
        etOrder.setUserId(userId);
        etOrder.setPaid("1");
        if(etOrderService.selectEtOrderList(etOrder).size()<1){//没有充值押金记录
            log.info("用户【{}】没有充值押金记录",userId);
            return false;
        }
        return true;
    }

    /**
     * 新增用户
     */
    @Override
    public String getNewUser(String powerStart, String powerEnd) {
        if(StringUtils.isNotEmpty(powerStart) && StringUtils.isNotEmpty(powerEnd)){
            return asUserMapper.getNewUser(powerStart,powerEnd);
        }
        return null;
    }

    /**
     * 总用户
     */
    @Override
    public String getTotalUser(String powerStart, String powerEnd) {
        if(StringUtils.isNotEmpty(powerStart) && StringUtils.isNotEmpty(powerEnd)){
            return asUserMapper.getTotalUser(powerStart,powerEnd);
        }
        return null;
    }

    /**
     * 租赁用户
     */
    @Override
    public String getLeaseUser(String powerStart, String powerEnd) {
        if(StringUtils.isNotEmpty(powerStart) && StringUtils.isNotEmpty(powerEnd)){
            return asUserMapper.getLeaseUser(powerStart,powerEnd);
        }
        return null;
    }

    /**
     * 绑定系统用户
     */
    @Override
    public int bandSystemUser(AsUser user) {
        return asUserMapper.updateUser(user);
    }

    /**
     * 实名认证
     */
    @Override
    public Object authentication(AuthenticationVo authenticationVo) {
        String idCard = authenticationVo.getIdCard();
        String realName = authenticationVo.getRealName();
        Object o = VerifyIdentityUtil.verifyIdentity(verifyUrl, appcode, idCard, realName);

        return o;
    }

}
